# frozen_string_literal: true

require 'uri'
require 'net/http'
require 'builder'
require 'active_support/all'
require 'dietary_dsl/bedca_api/constants'
require 'dietary_dsl/bedca_api/food_value'
require 'dietary_dsl/bedca_api/food_values'
require 'pp'

module DietaryDsl
  # Parte de la clase que interactúa con la base de datos
  class Food
    attr_reader :raw, :nombre, :name, :values

    include BEDCA

    @http = Net::HTTP.new(URL.host, URL.port)

    class << self
      attr_accessor :http
    end

    def self.request(xml)
      response = @http.request_post URL, XML_HEADER + xml, 'Content-Type' => 'text/xml'
      body = response.read_body
      return Hash.from_xml body
    rescue StandardError => e
      raise "El servidor ha devuelto un error: #{e.message}"
    end

    def self.sanitize_response(response)
      response.dig 'foodresponse', 'food'
    end

    def self.build_attributes(b, attributes)
      attributes
        .reject { |(key)| key == 'foodvalue' }
        .each do |(key)|
          b.atribute name: key
        end
    end

    def self.query(attributes = ATTRIBUTES)
      Builder::XmlMarkup.new.foodquery do |b|
        b.type level: 2
        b.selection do
          build_attributes(b, attributes)
        end
        yield b
      end
    end

    def self.where(b, field, relation, value)
      b.condition do
        b.cond1 do
          b.atribute1(name: ATTRIBUTES.key(field))
        end
        b.relation(type: relation)
        b.cond3(value)
      end
    end

    def self.find_by_and_relation(data, relation)
      xml = query do |b|
        data.each do |(key), value|
          where(b, key, relation, value)
        end
        yield b if block_given?
        order(b, :nombre)
      end

      data = sanitize_response request xml
      return nil if data.nil?

      Food.new(data)
    end

    def self.order(b, field, direction = 'ASC')
      b.order(ordtype: direction) do
        b.atribute3(name: ATTRIBUTES.key(field))
      end
    end

    def self.find(id)
      find_by(id: id)
    end

    def self.find_by(data)
      find_by_and_relation(data, RELATIONS[:equal])
    end

    def self.find_by_like(data)
      find_by_and_relation(data, RELATIONS[:like]) do |b|
        data.each do |(key), value|
          where(b, key, RELATIONS[:begin_with], value[0])
        end
      end
    end

    def initialize(values)
      @data = rehash(values)
      @groups = @data[:foodvalue]
                .group_by { |value| value['cg_descripcion'] }
                .each_with_object({}) do |(key, vals), hash|
                  hash[CATEGORIES[key]] = FoodValues.new(vals.first['cg_descripcion'], vals)
                end

      @data.delete(:foodvalue)
      @data[:groups] = @groups

      @general_values = download_general_values
    end

    def [](key)
      return @data[key].delete(',') if %i[nombre name].include?(key)
      @data[key]
    end

    def download_general_values
      body = (Food.http.request_get "http://www.bedca.net/cocoon/svg/response#{self[:id]}langes.svg").read_body
      parse_svg body
    end

    def parse_svg(svg)
      (svg.scan SVG_REGEX).each_with_object({}) do |value, hash|
        hash[COMPONENTS[value.last]] = value.first.delete(' ')
        hash
      end
    end
  end
end
